package net.za.darkskies.security.sasl;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Map;
import javax.security.auth.callback.*;
import javax.security.sasl.*;

final class ExternalServer
    implements SaslServer
{

	private CallbackHandler cbh;
	protected boolean completed;
	private String authorizationID;

	ExternalServer(CallbackHandler cbh)
        throws SaslException
    {
		this.cbh = cbh;
		completed = false;
	}

	public String getMechanismName()
	{
		return "EXTERNAL";
	}

	public boolean isComplete()
	{
		return completed;
	}

	public byte[] unwrap(byte incoming[], int offset, int len)
		throws SaslException
	{
		if (completed)
			throw new IllegalStateException("EXTERNAL supports neither integrity nor privacy");
		else
			throw new IllegalStateException("EXTERNAL authentication not completed");
	}

	public byte[] wrap(byte outgoing[], int offset, int len)
		throws SaslException
	{
		if (completed)
			throw new IllegalStateException("EXTERNAL supports neither integrity nor privacy");
		else
			throw new IllegalStateException("EXTERNAL authentication not completed");
	}

    public byte[] evaluateResponse(byte response[])
        throws SaslException
    {
        if(completed)
            throw new IllegalStateException("EXTERNAL authentication already completed");

		AuthorizeCallback authorizecallback = new AuthorizeCallback(null, null);
		try
		{
		cbh.handle(new Callback[] 
			{
				authorizecallback
			});
		}
		catch (UnsupportedCallbackException ex)
		{
			throw new SaslException("EXTERNAL authentication failed", ex);
		}
		catch (IOException ex)
		{
			throw new SaslException("EXTERNAL authentication failed", ex);
		}

		if(authorizecallback.isAuthorized()) 
		{
			completed = true;
			authorizationID = authorizecallback.getAuthorizedID();
		}
		else
			throw new SaslException("EXTERNAL authentication failed");
		
		return null;
		
	}

	public Object getNegotiatedProperty(String s)
	{
		if (completed)
		{
			if(s.equals(Sasl.QOP))
				return "auth";
			else
				return null;
		} 
		else
		{
			throw new IllegalStateException("SSPI authentication not completed");
		}
	}

	public String getAuthorizationID()
    {
        if (completed)
            return authorizationID;
        else
            throw new IllegalStateException("EXTERNAL authentication not completed");
    }

	public void dispose()
		throws SaslException
	{
	}

}
