package net.za.darkskies.security.sasl;

import java.util.*;
import javax.security.sasl.*;
import javax.security.auth.callback.CallbackHandler;

class SSPIImpl
	implements SaslClient, SaslServer
{

	static 
	{
		System.loadLibrary("SaslSspi");
		initIDs();
	}

	native static void initIDs();

	public static String join(String[] s, String delimiter) 
	{
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < s.length; i++)
		{
			buffer.append(s[i]);
			if (i < s.length - 1) 
			{
				buffer.append(delimiter);
			}
		}
		return buffer.toString();
	}
	
	// Client
	protected SSPIImpl(
		String[] mechanisms, 
		String authorizationId, 
		String protocol, 
		String serverName, 
		Map props, 
		CallbackHandler cbh)
	{
		clientInit(join(mechanisms, ","), protocol, serverName, props, cbh);
		completed = false;
		this.authorizationId = authorizationId;
	}

	native private void clientInit(
		String mechanisms, 
		String protocol, 
		String serverName, 
		Map props, 
		CallbackHandler cbh);

	native public static String[] getClientMechanismNames(Map map);

	// Server
	protected SSPIImpl(
		String mechanism, 
		String protocol, 
		String serverName,
		Map props, 
		CallbackHandler cbh) throws SaslException
	{
		serverInit(mechanism, protocol, serverName, props, cbh);
		completed = false;
	}

	native private void serverInit(
		String mechanism, 
		String protocol, 
		String serverName,
		Map props, 
		CallbackHandler cbh) throws SaslException;

	native public static String[] getServerMechanismNames(Map map);

	// Common
	native public String getMechanismName();

	protected long model;
	protected String authorizationId;
	protected boolean completed;

	public boolean isComplete()
	{
		return completed;
	}

	public byte[] unwrap(byte incoming[], int offset, int len)
		throws SaslException
	{
		if(completed)
			throw new IllegalStateException("SSPI Wrapper supports neither integrity nor privacy");
		else
			throw new IllegalStateException("SSPI authentication not completed");
	}

	public byte[] wrap(byte outgoing[], int offset, int len)
		throws SaslException
	{
		if(completed)
			throw new IllegalStateException("SSPI Wrapper supports neither integrity nor privacy");
		else
			throw new IllegalStateException("SSPI authentication not completed");
	}

	public Object getNegotiatedProperty(String s)
	{
		if(isComplete())
		{
			if(s.equals(Sasl.QOP))
				return "auth";
			else
				return null;
		} 
		else
		{
			throw new IllegalStateException("SSPI authentication not completed");
		}
	}

	public boolean hasInitialResponse()
	{
		return true;
	}

	native public byte[] evaluateChallenge(byte challenge[])
		throws SaslException;

	native public byte[] evaluateResponse(byte response[])
		throws SaslException;

	native public String getAuthorizationID();

	public void dispose()
		throws SaslException
	{
		clearPassword();
	}

	native protected void clearPassword();

	protected void finalize()
	{
		clearPassword();
	}

}
